class TalltyImportExport::ExportPayload::Cell
  attr_accessor :cell_cluster, :value, :header, :parent_path

  # cell_cluster 是表格纵向 集合
  # cell_the_same_line 是表格横向 集合
  def initialize header, value, context, parent_path=[], &value_handler
    @value = value
    @header = header
    @value_handler = value_handler

    # context
    @context = context

    value.seq_chain.each do |s|
      unless (@context[:header_seq_to_value_seq_to_value][header.seq])
        @context[:header_seq_to_value_seq_to_value][header.seq] = {}
      end
      @context[:header_seq_to_value_seq_to_value][header.seq][s] = value # NOTE: value 对象
    end

    # parent
    @parent_path = parent_path

    @cell_cluster = []
  end

  def seq
    @seq ||= SecureRandom.uuid
  end

  def depth
    @parent_path.count
  end

  def header_chain_seqs
    [*@parent_path, self].map { |cell| cell.header.cluster_seqs }.reduce(:concat)
  end

  def divise
    if @header.children.present?
      new_cells = @header.children.map do |child_header|
        if (child_header.children.present?)
          val = @value.value[child_header.key.to_sym]
          order = 0
          val.map do |val|
            cell = TalltyImportExport::ExportPayload::Cell.new(
              child_header,
              TalltyImportExport::ExportPayload::Value.new(val, @value.chain, inherited: true),
              @context,
              [*self.parent_path, self],
              &@value_handler
            )
            order += 1
            cell
          end
        else
          val = @value_handler.call(@value.value, child_header.as_json.symbolize_keys)
          TalltyImportExport::ExportPayload::Cell.new(
            child_header,
            TalltyImportExport::ExportPayload::Value.new(val, @value.chain),
            @context,
            [*self.parent_path, self],
            &@value_handler
          )
        end
      end.flatten
      new_cells.each(&:divise)
      @cell_cluster.concat(new_cells.map(&:cell_cluster))
    else
      @cell_cluster = [self]
    end

    self
  end

  def grow_to_line
    @context[:flatten_headers].
      select do |header|
        header_chain_seqs.include?(header.seq)
      end.
      map do |header|
        target_value = nil

        @value.seq_chain.uniq.reverse.each do |seq|
          next if target_value
          # seq = @value.seq
          if @context[:header_seq_to_value_seq_to_value][header.seq]&.[](seq)
            target_value = @context[:header_seq_to_value_seq_to_value][header.seq][seq]
          end
      end

      target_value
    end
  end
end
